@@ -290,11 +290,11 @@ notify_socket_cb (GNetworkIpMulticast * multicast, GParamSpec * pspec, gpointer
     {
     case GNETWORK_PROTOCOL_IPv4:
       success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IP, IP_MULTICAST_TTL,
-			     (gpointer) value, sizeof (value)) >= 0);
+			     GINT_TO_POINTER (value), sizeof (value)) >= 0);
       break;
     case GNETWORK_PROTOCOL_IPv6:
       success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
-			     (gpointer) value, sizeof (value)) >= 0);
+			     GINT_TO_POINTER (value), sizeof (value)) >= 0);
       break;
     default:
       g_assert_not_reached ();
@@ -325,11 +325,11 @@ notify_socket_cb (GNetworkIpMulticast * multicast, GParamSpec * pspec, gpointer
     {
     case GNETWORK_PROTOCOL_IPv4:
       success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IP, IP_MULTICAST_LOOP,
-			     (gpointer) value, sizeof (value)) >= 0);
+			     GINT_TO_POINTER (value), sizeof (value)) >= 0);
       break;
     case GNETWORK_PROTOCOL_IPv6:
       success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IPV6, IPV6_MULTICAST_LOOP,
-			     (gpointer) value, sizeof (value)) >= 0);
+			     GINT_TO_POINTER (value), sizeof (value)) >= 0);
       break;
     default:
       g_assert_not_reached ();
@@ -405,11 +405,11 @@ gnetwork_ip_multicast_set_property (GObject * object, guint property, const GVal
 	      {
 	      case GNETWORK_PROTOCOL_IPv4:
 		success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IP, IP_MULTICAST_TTL,
-				       (gpointer) ttl, sizeof (ttl)) >= 0);
+				       GINT_TO_POINTER (ttl), sizeof (ttl)) >= 0);
 		break;
 	      case GNETWORK_PROTOCOL_IPv6:
 		success = (setsockopt (GPOINTER_TO_INT (sockfd), IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
-				       (gpointer) ttl, sizeof (ttl)) >= 0);
+				       GINT_TO_POINTER (ttl), sizeof (ttl)) >= 0);
 		break;
 	      default:
 		g_assert_not_reached ();
