@@ -1661,7 +1661,7 @@ gnetwork_tcp_proxy_get_use_proxy (GNetworkTcpProxyType type, const gchar * addre
       for (exceptions = gconf_client_get_list (client, PROXY_EXCEPTIONS_KEY, GCONF_VALUE_LIST,
 					       NULL), retval = TRUE;
 	   exceptions != NULL && retval == TRUE;
-	   exceptions = g_slist_remove_link (exceptions, exceptions),
+	   exceptions = g_slist_delete_link (exceptions, exceptions),
 	   retval = g_pattern_match_simple (exceptions->data, address));
 
       /* If it's not covered by a rule, check that the proxy host is != NULL */
