@@ -399,6 +399,7 @@ gnetwork_tcp_server_open (GNetworkTcpServer * server)
       switch (errno)
 	{
 	case EINVAL:
+	case EADDRINUSE:
 	  error = g_error_new (GNETWORK_SERVER_ERROR, GNETWORK_SERVER_ERROR_ALREADY_EXISTS,
 			       _("The service could not be started because there is already "
 				 "another service using port %d."), server->_priv->port);
@@ -485,7 +486,8 @@ gnetwork_tcp_server_close (GNetworkTcpServer * server)
   g_object_notify (G_OBJECT (server), "status");
 
   for (; server->_priv->connections != NULL;
-       g_slist_remove_link (server->_priv->connections, server->_priv->connections))
+       server->_priv->connections = g_slist_delete_link (server->_priv->connections,
+							 server->_priv->connections))
     {
       if (server->_priv->close_children)
 	{
