@@ -189,6 +189,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
       udp->_priv->sockfd = -1;
 
       udp->_priv->dgram_status = GNETWORK_DATAGRAM_CLOSED;
+      g_object_notify (G_OBJECT (udp), "socket");
       g_object_notify (G_OBJECT (udp), "status");
       return FALSE;
     }
@@ -243,6 +244,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
 	      udp->_priv->sockfd = -1;
 
 	      udp->_priv->dgram_status = GNETWORK_DATAGRAM_CLOSED;
+      	      g_object_notify (G_OBJECT (udp), "socket");
 	      g_object_notify (G_OBJECT (udp), "status");
 	      retval = FALSE;
 	      break;
@@ -281,7 +283,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
 	  GValue value = { 0 };
 
 	  item = udp->_priv->buffer->data;
-	  udp->_priv->buffer = g_slist_remove_link (udp->_priv->buffer, udp->_priv->buffer);
+	  udp->_priv->buffer = g_slist_delete_link (udp->_priv->buffer, udp->_priv->buffer);
 
 	  sa = _gnetwork_ip_address_to_sockaddr (&(item->target->ip_address), item->target->port,
 						 &sa_size);
@@ -319,6 +321,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
 		  udp->_priv->sockfd = -1;
 
 		  udp->_priv->dgram_status = GNETWORK_DATAGRAM_CLOSED;
+		  g_object_notify (G_OBJECT (udp), "socket");
 		  g_object_notify (G_OBJECT (udp), "status");
 		  retval = FALSE;
 		  break;
@@ -356,6 +359,7 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
 	      udp->_priv->sockfd = -1;
 
 	      udp->_priv->dgram_status = GNETWORK_DATAGRAM_CLOSED;
+      	      g_object_notify (G_OBJECT (udp), "socket");
 	      g_object_notify (G_OBJECT (udp), "status");
 	      retval = FALSE;
 	    }
@@ -715,19 +719,34 @@ gnetwork_udp_datagram_close (GNetworkUdpDatagram * udp)
   if (udp->_priv->dgram_status <= GNETWORK_DATAGRAM_CLOSED)
     return;
 
+  if (udp->_priv->source_id)
+    {
+      gnetwork_thread_source_remove (udp->_priv->source_id);
+      udp->_priv->source_cond = 0;
+      udp->_priv->source_id = 0;
+    }
+
   if (udp->_priv->channel != NULL)
     {
+      g_io_channel_shutdown (udp->_priv->channel, FALSE, NULL);
       g_io_channel_unref (udp->_priv->channel);
       udp->_priv->channel = NULL;
     }
-
-  if (udp->_priv->sockfd > 0)
+  else if (udp->_priv->sockfd > 0)
     {
       shutdown (udp->_priv->sockfd, SHUT_RDWR);
       close (udp->_priv->sockfd);
-      udp->_priv->sockfd = -1;
     }
 
+  while (udp->_priv->buffer != NULL)
+    {
+      free_buffer_item (udp->_priv->buffer->data);
+
+      udp->_priv->buffer = g_slist_delete_link (udp->_priv->buffer,
+						udp->_priv->buffer);
+    }
+
+  udp->_priv->sockfd = -1;
   udp->_priv->dgram_status = GNETWORK_DATAGRAM_CLOSED;
   g_object_notify (G_OBJECT (udp), "socket");
   g_object_notify (G_OBJECT (udp), "status");
@@ -918,12 +937,12 @@ gnetwork_udp_datagram_set_property (GObject * object, guint property, const GVal
 	    switch (_gnetwork_get_socket_protocol (GINT_TO_POINTER (udp->_priv->sockfd)))
 	      {
 	      case GNETWORK_PROTOCOL_IPv4:
-		success = (setsockopt (udp->_priv->sockfd, IPPROTO_IP, IP_TTL, (gpointer) ttl,
+		success = (setsockopt (udp->_priv->sockfd, IPPROTO_IP, IP_TTL, GINT_TO_POINTER (ttl),
 				       sizeof (ttl)) >= 0);
 		break;
 	      case GNETWORK_PROTOCOL_IPv6:
 		success = (setsockopt (udp->_priv->sockfd, IPPROTO_IPV6, IPV6_UNICAST_HOPS,
-				       (gpointer) ttl, sizeof (ttl)) >= 0);
+				       GINT_TO_POINTER (ttl), sizeof (ttl)) >= 0);
 		break;
 	      default:
 		g_assert_not_reached ();
@@ -967,7 +986,7 @@ gnetwork_udp_datagram_set_property (GObject * object, guint property, const GVal
 	  return;
 
 	if (udp->_priv->sockfd > 0 &&
-	    setsockopt (udp->_priv->sockfd, SOL_SOCKET, SO_BROADCAST, (gpointer) broadcast,
+	    setsockopt (udp->_priv->sockfd, SOL_SOCKET, SO_BROADCAST, GINT_TO_POINTER (broadcast),
 			sizeof (broadcast)) == -1)
 	  {
 	    if (udp->_priv->port != 0)
@@ -1065,7 +1084,7 @@ gnetwork_udp_datagram_dispose (GObject * object)
   GNetworkUdpDatagram *udp = GNETWORK_UDP_DATAGRAM (object);
 
   if (udp->_priv->dgram_status > GNETWORK_DATAGRAM_CLOSED)
-    gnetwork_udp_datagram_close (udp);
+    gnetwork_datagram_close (udp);
 
   gnetwork_interface_info_unref (udp->_priv->interface_info);
 
@@ -1106,7 +1125,7 @@ gnetwork_udp_datagram_class_init (GNetworkUdpDatagramClass * class)
   g_object_class_install_property (object_class, BROADCAST,
 				   g_param_spec_boolean ("broadcast", _("Allow Broadcasting"),
 							 _("Whether or not to allow broadcasting "
-							   "through this datagram."), FALSE,
+							   "through this socket."), FALSE,
 							 (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
   g_object_class_install_property (object_class, INTERFACE,
 				   g_param_spec_string ("interface", _("Local Interface"),
